﻿--- Constructors & Utilities ---

local addonName, addonTable = ...
local C = addonTable.C

local framepool = {}

addonTable.CreateDialogFrame = function(parent, enablemouse)
 local f  = CreateFrame("FRAME")
 f:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background", 
  edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
  tile = true, tileSize = 32, edgeSize = 32, 
  insets = { left = 11, right = 12, top = 12, bottom = 11 }});
 f:SetBackdropColor(0,0,0,1)
 f:SetParent(parent)
 f:SetPoint("CENTER")
 f:SetFrameStrata("HIGH")
 if enablemouse then
  f:EnableMouse(true)
  f:SetMovable(true)
  f:RegisterForDrag("LeftButton")
  f:SetScript("OnDragStart", function(self)
		 self:StartMoving()
   end)
  f:SetScript("OnDragStop", function(self) 
	 self:StopMovingOrSizing()
   end)
 end

 return f
end

addonTable.CreateSimpleButton = function(parent)
 local b = CreateFrame("BUTTON", "Wowzzle_btn", parent, "OptionsListButtonTemplate")
 b:SetWidth(150)
 b:SetHeight(30)
 return b
end

--- grid generator ---
local Iletters = {"A","B","C","D","E","F","G","H","J","K","I","L","M",
                  "N","O","P","Q","R","S","T","U","V","W","X","Y","Z"}
local rnd = math.random
local upp = string.upper
local grid = {} -- the 5x5 grid
local used = {} -- list of used cells for the current word
local sel -- selected xy

local function getSel()
 return grid[(sel[2] - 1) * 5 + sel[1]]
end

local function setPicked(x,y)
 getSel().picked = true
 table.insert(used, {sel[1],sel[2]})
end

local function pickNear(s)

 local function isUsed(cell)
  table.foreach(used, function(k,v) 
    if v[1] == cell[1] and v[2] == cell[2] then
     return true
    end
   end)
  return false
 end

 local function checkGrid(x,y)
  if y >= 1 and x >= 1 and y <= 5 and x <= 5 then
   if (not grid[(y - 1) * 5 + x].picked) or 
      (s == grid[(y - 1) * 5 + x].s and not isUsed({x,y})) then
    return true
   end
  end
  return false
 end

 local allowed = {}
 local x,y

 for x = -1,1 do
  for y = -1,1 do
   if not (x == 0 and y == 0) then
    if checkGrid(x + sel[1], y + sel[2]) then
     table.insert(allowed, {x + sel[1], y + sel[2]})
    end
   end
  end
 end
 
 if table.getn(allowed) == 0 then
  return false
 else
  local i = rnd(table.getn(allowed))
  sel[1] = allowed[i][1]
  sel[2] = allowed[i][2]
  return true
 end
end

local function pickRndWord(length)
 local result = ""
 while strlen(result) ~= length do
  local l = Iletters[rnd(24)]
  local size = table.getn(addonTable.wowzzle_database[l])
  result = addonTable.wowzzle_database[l][rnd(size)]
 end

 return result
end

local function placeLetter(s)
 getSel().s = s
 setPicked()
end

local function getRndLetter()
 local l = Iletters[rnd(24)]
 if l == "J" or l == "K" or l == "Z" or l == "Q" or l == "H" or l == "W" then
  l = Iletters[rnd(24)]
 end
 return l
end

addonTable.GenerateGrid = function()
 local w,i,result

 wipe(grid)
 for i = 1,25 do
  table.insert(grid, {s = "#", picked = false})
 end

 local vocals = {"A","E","I","O","U"}
 local l,j
 for j = 1,5 do
  if vocals[j] then -- place the 5 vocals once
   sel = {rnd(5),rnd(5)}
   while getSel().picked do
    sel = {rnd(5),rnd(5)}
   end
   getSel().x = vocals[j]
   setPicked()
   table.remove(vocals,j)
  end
 end
 
 local function newWord()
  wipe(used)
  local c = 200
  sel = {rnd(5),rnd(5)}
  while c > 0 and getSel().picked do
   sel = {rnd(5),rnd(5)}
   c = c - 1
  end

  w = pickRndWord(rnd(4)+4)
  i = strlen(w)
 end
 
 local counter = 1
 local tries = 10
 newWord()

 while tries > 0 do
  if pickNear(w:sub(counter,counter)) then
   placeLetter(w:sub(counter,counter))
   counter = counter + 1
   if counter > i then
    newWord()
    counter = 1
   end
  else
   if tries > 0 then
    newWord()
    counter = 1
    tries = tries - 1
   end
  end
 end
 
 result = ""
 table.foreach(grid, function(k,v)
   if v.s == "#" then
    v.s = getRndLetter()
   end
   result = result..upp(v.s)
  end)
 return result
end


----------------------